@@ -1,6 +1,7 @@
+"_PACKAGE"
 #' clusterability: a package to perform tests of clusterability
 #'
-# Copyright (C) 2020  Zachariah Neville, Naomi Brownstein, Andreas Adolfsson, Margareta Ackerman
+# Copyright (C) 2025  Zachariah Neville, Naomi Brownstein, Andreas Adolfsson, Margareta Ackerman
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -139,6 +140,6 @@
 #' print(cars_silvdist)
 #'
 #'
-#' @docType package
+#'
 #' @name clusterability
 NULL
